#!/usr/bin/perl -w

#    xxd.pm : print a dump in Hex and ASCII
#    Copyright (C) 2001  Ludovic Rousseau <ludovic.rousseau@free.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package xxd;

use strict;
require Exporter;

use vars qw($VERSION @ISA @EXPORT);

# example:
#print xxd(0x09E0, "12345678");
#print xxd_plus(0, "azertyuiopqsdfghjklm", 2);

$VERSION = '0.2';
@ISA = qw(Exporter);
@EXPORT = qw(xxd xxd_plus);

#
# print a memory dump
#
# IN:
#	address (ex: 0x09E0)
#	bloc (ex: "123456789ABCDEFGHIJKLMNOP")
# OUT:
#	09E0 31 32 33 34 35 36 37 38 39 41 42 43 44 45 46 47  123456789ABCDEFG    
#	09F0 48 49 4A 4B 4C 4D 4E 4F 50                       HIJKLMNOP           
sub xxd($$)
{
	my $adr = shift;
	my $bin = shift;

	return xxd_plus($adr, $bin, 1);
}

#
# change the address factor (hack for Carte Bleue)
#
sub xxd_plus($$$)
{
	my $adr = shift;
	my $bin = shift;
	my $factor = shift;
	my $text = "";
	my $l;

	while (($l = length $bin) >= 16)
	{
		$text .= &xxd_line($adr, $bin, 16);
		$text .= "\n";

		$bin = substr $bin, 16;
		$adr += 16 * $factor;
	}

	if ($l > 0)
	{
		$text .= &xxd_line($adr, $bin, $l);
		$text .= "\n";
	}

	return $text;
}

sub xxd_line($$$)
{
	my $adr = shift;
	my $bin = shift;
	my $len = shift;
	my ($line, $c, $i);

	# address
	$line = sprintf("%04X", $adr);

	# fill with ' '
	$line .= " "x(1+16*3+1+16);

	for ($i=0; $i<$len; $i++)
	{
		$c = ord (substr $bin, $i, 1);
		substr $line, 4+1+($i*3), 2, sprintf("%02X", $c);

		if ($c > 31)
		{
			substr $line, 4+1+16*3+1+$i, 1, chr $c;
		}
		else
		{
			substr $line, 4+1+16*3+1+$i, 1, ".";
		}
	}

	return $line;
}

1;

__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

xxd - Perl extension to dump a string in hex/ascii format

=head1 SYNOPSIS

 use xxd;

 xxd(ADDR, CONTENTS);
 xxd_plus(ADDR, CONTENTS, FACTOR);

=head1 DESCRIPTION

Returns a dump of CONTENTS in hex and ASCII like the L<xxd(1)> command. The
address to use is ADDR.

The FACTOR is used to increase ADDR of more than 16 for each line.
xxd(ADDR, CONTENTS) is a call to xxd_plus(ADDR, CONTENTS, 1).

=head1 EXAMPLE

 print xxd(0x09E0, "12345678");

prints S<09E0 31 32 33 34 35 36 37 38                          12345678>

 print xxd_plus(0, "azertyuiopqsdfghjklm", 2);

prints
S<0000 61 7A 65 72 74 79 75 69 6F 70 71 73 64 66 67 68  azertyuiopqsdfgh>
S<0020 6A 6B 6C 6D                                      jklm>

=head1 BUGS

None known.

=head1 AUTHORS

Ludovic Rousseau (ludovic.rousseau@free.fr)

=head1 SEE ALSO

perl(1p), xxd(1)

=cut

